function [ z ] = itsolve_iigd(r, lam, x, itmax, tol, lsolver, A);
%
% solve the eq.  (A - lam B) z = -r + tau*x  using 
% iterative solver tk_bicgstab and tk_gmres, 
% with different tol and itmax.
% 
% Matlab built-in linear solvers are not used because for r small,
% the built-in solvers often refuse to iterate or often return
% a zero vector, which makes the continued computation not very 
% meaningful.
%

  switch lsolver 
   case {'bicgstab'},
    [xtmp]= tk_bicgstab(@Kiigd, x, tol, itmax, lam, A);
    z = tk_bicgstab(@Kiigd, r, tol, itmax, lam, A);
   case {'gmres'},
    [xtmp]= tk_gmres(@Kiigd, x,  tol, itmax, lam, A);
    z = tk_gmres(@Kiigd, r, tol, itmax, lam, A);
  end
  
  tau = (x'*xtmp);
  nrmx = norm(xtmp); 
  nrmz=norm(z);

  if (tau ~= 0  &  nrmx ~=0  & nrmz ~=0),
    tau = x'*z/ tau;  
    z = tau*xtmp - z;
  else
    fprintf('***** in itsolve_iigd, solver=%s\n', lsolver)
    fprintf('***** <x, xtmp>=%e,  nrmx=%e,  nrmz=%e\n', tau, nrmx, nrmz);
    error('zero solution vector encountered')
  end

  
  
  
function [y]=Kiigd(r, lam, A)
%
% compute  y = (A-lamI)*r
%
  y = user_Hx(r, A);
  y = y - lam*r;

