function [t]=solveA(r, rho, x, les_itmax, les_tol, les_solver, eigV, A);
%
% this handles the case that A is a script for the matrix-vector product
%
  global is_alpha  alpha
  
  switch (les_solver)
   case {'infl-bicgs', 'infl-gmres', 'infl-minres', 'rand-gmres', ...
	 'rand-bicgs','cons-gmres','cons-bicgs','consm-gmres','consm-bicgs'},
    if (is_alpha),
      t=modrqi(r,rho,x,les_itmax,les_tol, les_solver, alpha, A);
    else
      t=modrqi(r,rho,x,les_itmax,les_tol, les_solver, A);
    end
    
   case {31, 'jd-bicgs'}      % iterative solve by JD-bicgstab
     t=jd_solve(r,rho,x,les_itmax,les_tol,'bicgstab',eigV, A);
   case {32, 'jd-bicgs-fp'}  % iterative solve by JD-bicgstab-FP
     t=jd_solve(r,rho,x,les_itmax,les_tol,'bicgstabFP',eigV, A);
	 
   case {71, 'jd-minres'}     %iterative solve by JD-minres (symmetric case)
     if (Matsymm==1),
       t=jd_solve(r, rho, x, les_itmax, les_tol,'minres',eigV, A);
     else
       error('MINRES works only for symmetric problems')
     end
   case {72, 'jd-minres-fp'}  %iterative solve by JD-minres-FP (symmetric case)
     if (Matsymm==1),
       t=jd_solve(r, rho, x, les_itmax, les_tol,'minresFP',eigV, A);
     else
       error('MINRES works only for symmetric problems')
     end 
            
   case {41, 'jd-gmres'}      % iterative solve by JD-gmres
     t=jd_solve(r, rho, x, les_itmax, les_tol,'gmres', eigV, A);
   case {42, 'jd-gmres-fp'}   % iterative solve by JD-gmres-FP
     t=jd_solve(r, rho, x, les_itmax, les_tol,'gmresFP',eigV, A);
      
   case {51, 'jdm-bicgs'}     % iterative solve by JDm-bicgstab
     t=jdm_solve(r, rho, x,les_itmax,les_tol,'bicgstab',eigV, A);
   case {52, 'jdm-bicgs-fp'}  % iterative solve by JDm-bicgstab-FP
     t=jdm_solve(r, rho, x,les_itmax,les_tol,'bicgstabFP',eigV, A);
	 
   case {61, 'jdm-gmres'}     % iterative solve by JDm-gmres
     t=jdm_solve(r, rho, x, les_itmax, les_tol,'gmres',eigV, A);
   case {62, 'jdm-gmres-fp'}  % iterative solve by JDm-gmres-FP
     t=jdm_solve(r, rho, x, les_itmax, les_tol,'gmresFP',eigV, A);

   case {21, 'rqi-gmres'}     % iterative solve by RQI-gmres
     t=itsolve_rqi(r,rho,x,les_itmax,les_tol,'gmres');
   case {22, 'rqi-gmres-fp'}  % iterative solve by RQI-gmres-FP
     t=itsolve_rqi(r,rho,x,les_itmax,les_tol,'gmres', ...
		   eigV, A);	
   case {23, 'rqi-bicgs'}     % iterative solve by RQI-bicgstab
     t=itsolve_rqi(r,rho,x,les_itmax,les_tol,'bicgstab');
   case {24, 'rqi-bicgs-fp'}  % iterative solve by RQI-bicgstab-FP
     t=itsolve_rqi(r,rho,x,les_itmax,les_tol,'bicgstab', ...
		   eigV, A);
   case {25, 'rqi-minres'}    %iterative solve by RQI-minres (symmetric case)
     if (Matsymm==1),
       t=itsolve_rqi(r,rho,x,les_itmax,les_tol,'minres');
     else
       error('MINRES works only for symmetric problems')
     end 
   case {26, 'rqi-minres-fp'} %iterative solve by RQI-minres-FP (symmetric case)
     if (Matsymm==1),
       t=itsolve_rqi(r,rho,x,les_itmax,les_tol,'minres', ...
		     eigV, A);
     else
       error('MINRES works only for symmetric problems')
     end 
      
   otherwise
     error('Input les_solver type to  gdav.m  does not exist')
  end